import time

import bs4
import keyring
import requests

from .constants import *


def submit(args):
    contest, problem_letter = translate_problem_name(args.problem)

    with requests.Session() as s:
        site = s.get('https://codeforces.com/enter')
        soup = bs4.BeautifulSoup(site.content, 'html.parser')
        csrf_token = soup.select_one('.csrf-token')['data-csrf']
        username = get_username()
        password = keyring.get_password('codeforces-toolbox', username)
        login_data = {'csrf_token': csrf_token, 'action': 'enter', 'handleOrEmail': username, 'password': password}
        login_response = s.post('https://codeforces.com/enter', data=login_data)
        try:
            login_response.raise_for_status()
            if bs4.BeautifulSoup(login_response.content, 'html.parser').select_one('for__password') is not None:
                raise Exception('Invalid username or password.')
        except requests.HTTPError:
            print(error_style("Something went wrong while logging in."))
            sys.exit()
        except Exception as e:
            print(error_style(e))
            sys.exit()

        site = s.get(f'https://codeforces.com/contest/{contest}/submit')
        soup = bs4.BeautifulSoup(site.content, 'html.parser')
        csrf_token = soup.select_one('.csrf-token')['data-csrf']
        language = get_language()
        with open(os.path.join(os.getcwd(), f'{contest}{problem_letter}.{language.ext}')) as f:
            solution = f.read()
        submit_data = {'csrf_token': csrf_token, 'submittedProblemIndex': problem_letter, 'programTypeId': language.n,
                       'source': solution}
        submit_response = s.post(f'https://codeforces.com/contest/{contest}/submit', data=submit_data)
        try:
            submit_response.raise_for_status()
        except requests.HTTPError:
            print(error_style("Something went wrong while submitting."))
            sys.exit()
        print('Solution has been submitted.')

        print('Verdict:')
        while True:
            site = s.get(f'https://codeforces.com/submissions/{username}')
            soup = bs4.BeautifulSoup(site.content, 'html.parser')
            v = soup.select_one('div.datatable table tr:nth-child(2) td.status-verdict-cell').text.strip()
            print('\033[F\033[K', end='')
            if v.startswith('Running') or v == 'In queue':
                print('Verdict: ' + v)
            elif v == 'Accepted':
                print('Verdict: ' + positive_style(v))
                break
            else:
                print('Verdict: ' + negative_style(v))
                break
            time.sleep(.05)



#!/usr/bin/env python

import argparse

from cft import __version__
from cft.utils.config import config
from cft.utils.constants import *
from cft.utils.race import race
from cft.utils.submit import submit
from cft.utils.test import test


def main():
    os.system('')    # enable colors in Windows cmd
    parser = argparse.ArgumentParser(prog='cft', description=neutral_style('----- Codeforces Toolbox -----'),
                                     epilog='Wish you high ratings!')
    parser.add_argument('-v', '--version', action='version', version=neutral_style(f'Codeforces Toolbox {__version__}'),
                        help='show version number')
    subparsers = parser.add_subparsers()

    config_parser = subparsers.add_parser('config', help='change configuration of the cft')
    config_parser.set_defaults(func=config)

    race_parser = subparsers.add_parser('race', help='create folder and solution files based on template')
    race_parser.add_argument('contest', type=str, help='contest id')
    race_parser.set_defaults(func=race)

    test_parser = subparsers.add_parser('test', help='test solution file')
    test_parser.add_argument('problem', type=str, help='problem id in the form "A" or "1234A"')
    test_parser.add_argument('-d', '--download', action='store_true', help='force download test from CF')
    test_parser.set_defaults(func=test)

    submit_parser = subparsers.add_parser('submit', help='submit solution')
    submit_parser.add_argument('problem', type=str, help='problem id in the form "A" or "1234A"')
    submit_parser.set_defaults(func=submit)

    args = parser.parse_args(None if sys.argv[1:] else ['-h'])
    args.func(args)



#!/usr/bin/env python

from setuptools import setup
from cft import __version__

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()
with open('requirements.txt') as f:
    requirements = f.read().split('\n')

setup(
    name='codeforces-toolbox',
    version=__version__,
    description='Codeforces CLI that makes writing contests more efficient.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Michał Dobranowski',
    author_email='mdbrnowski@gmail.com',
    url='https://github.com/mdbrnowski/codeforces-toolbox',
    packages=['cft', 'cft.utils'],
    classifiers=['Development Status :: 5 - Production/Stable',
                 'Environment :: Console',
                 'License :: OSI Approved :: MIT License',
                 'Operating System :: MacOS',
                 'Operating System :: Microsoft :: Windows',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python',
                 'Topic :: Terminals'],
    license='MIT',
    keywords=['cli', 'competitive-programming', 'codeforces'],
    install_requires=requirements,
    python_requires='>=3.7',
    entry_points={'console_scripts': ['cft=cft.main:main']},
)
